#include <time.h>
#include <rfb/rfbclient.h>
#include "screentoimage.h"

static char *sc_get_password(rfbClient* client) {
	char *pw = getenv("VNCPASSWD");
	if (!pw) {
		rfbClientLog("Please provide VNCPASSWD environment variable\n");
		exit(1);
	}
	return strdup(pw);
}

static void sc_got_frame_buffer_update(rfbClient* client, int x, int y, int w, int h) {
  rfbClientLog("Received an update for %d,%d,%d,%d.\n",x,y,w,h);
}

static void sc_do_screenshot(rfbClient* client, const char *filename) {
	screentoimage_init();
	int rc = screentoimage_write(client, filename);
	if (rc == -1) {
		rfbClientLog("Failed to write screenshot\n");
	} else {
		rfbClientLog("screenshot writed\n");
	}
	screentoimage_finalize();
}

int main(int argc, char **argv) {
	const char *filename = getenv("VNCSCREENSHOT_FILENAME");
	
	if (!filename) {
		rfbClientLog("Please provide VNCSCREENSHOT_FILENAME environment variable\n", argv[0]);
		return 1;
	}	
	
	rfbClient* client = rfbGetClient(8,3,4);

	client->GetPassword          = sc_get_password;
	client->GotFrameBufferUpdate = sc_got_frame_buffer_update;

	if (!rfbInitClient(client,&argc,argv))
		return 2;

	time_t start = time(NULL);

	int got_error = 0;

	while (time(NULL) - start < 2) {
		int rc = WaitForMessage(client, 1000);
		if (rc < 0) {
			got_error = 1;
			break;
		}
		if (rc == 0)
			continue;
		if(!HandleRFBServerMessage(client)) {
			got_error = 1;
			break;
		}
	}

	if (!got_error) {
		sc_do_screenshot(client, filename);
	}

	rfbClientCleanup(client);

	exit(got_error ? 3 : 0);
}
