require 'socket'
require 'optparse'
require 'net/http'
require 'network-projector/vnc/share'

module NetworkProjector
	class NPClient
		def run!
			options = {
				:passwd=>'unsecure',
				:host=>'localhost',
				:port=>'9495',
				:display=>":0",
				:user=>Socket.gethostname
			}

			parser = OptionParser.new

			parser.on('--help', 'displays usage information') do
				puts parser
				exit
			end

			parser.on('-h=HOST', '--host=HOST', "Projector host (#{options[:host]})") do |v|
				options[:host] = v
			end

			parser.on('-d=DISPLAY', '--display=HOST', "Local display to share (#{options[:display]})") do |v|
				options[:display] = v
			end

			parser.on('-u=USER', '--user=USER', "User label (#{options[:label]})") do |v|
				options[:user] = v
			end

			parser.on('-p=PASSWD', '--passwd=PASSWD', 'Vnc password') do |v|
				options[:passwd] = v
			end

			parser.on('--clip=CLIP', 'x11vnc clip option (ex: xinerama0)') do |v|
				options[:clip] = v
			end

			begin
				parser.parse($*)
			rescue OptionParser::ParseError
				puts $!
				exit
			end

			@user = options[:user]

			@vs = VncShare.new options

			@vs.run

			if not @vs.started? then
				puts "Can't start, leaving"
				return
			end
			begin
				@http = Net::HTTP.new(options[:host], options[:port])
				connect
				puts "Ready for connection"
				@vs.join
			rescue SystemCallError => e
				puts "Connection to #{options[:host]}:#{options[:port]} failed : #{e}"
				@vs.stop
			rescue Interrupt
				@vs.stop
			ensure
				disconnect
			end
			puts "Leaving"
		end

		def connect
				request = Net::HTTP::Post.new("/connect")
				request.set_form_data({"user" => @user, "vnc[port]" => @vs.port, "vnc[passwd]" => @vs.passwd})
				response = @http.request(request)
				@id = response.body
				puts "Connected as #{@id}"
		end

		def disconnect
				request = Net::HTTP::Post.new("/disconnect")
				request.set_form_data({"id" => @id})
				response = @http.request(request)
				puts "Disconnect response (#{response.code}) : #{response.body}" unless response.code == "200"
		end
	end
end
